% Code to run generalized beta and Gaussian density through
% JMCB histogram answers (to question 16) and to compute moments nonparametrically.
% Creates Figure 1 in Galvao and Mitchell main paper. Also computes various
% statistics referenced in the paper.

% Import data - variable description below
% ResponseId - unique identifier of survey respondent
% prob - probability assigned to a particular bin
% cum_prob - CDF value at a particular bin
% n_bins - total number of bins with assigned probability
% min - minimum of bin's interval
% max - maximum of bin's interval
% l - global minimum of distribution's support, where known
% r - global maximum of distribution's support, where known
clear all
%data = readtable('beta_estimation_input.csv');


% Load in histogram data. Each row is a new respondent. The cols indicate
% the probabilities in each bin. The bin intervals are defined below and
% also indicated in question 16 (see the main paper).
filename = 'Public Survey data analysis.xlsx';
sheet = 'Sheet1';
xlRange = 'A1:H4201';

data = xlsread(filename,sheet,xlRange)


% select this if you want to extract only the control group
control=data(data(:,8)==1,:);
control=control(:,1:7);

%data=control;
data=data(:,[1:7]);
q16_1=data(:,1);q16_2=data(:,2);q16_3=data(:,3);q16_4=data(:,4);q16_5=data(:,5);q16_6=data(:,6);q16_7=data(:,7);
data=flip(data,2);
data_full=data;

cum_prob=cumsum(data')';

% find multiples as in Wallis paper
% y = data(mod(data(1,:),10)==0);


% delete respondents for whom we have no data
data(cum_prob(:, 7)== 0, :)= [];
cum_prob(cum_prob(:, 7)== 0, :)= [];


binn=data==0; binn=abs(binn-1);

% Find number of bins probabilities are assigned to
n_bins=zeros(size(data,1),1);
mini=zeros(size(data,1),1);
maxi=zeros(size(data,1),1);
xi=zeros(7,1);
xi(1)=20.5;
xi(2)=21;
xi(3)=21.5;
xi(4)=22;
xi(5)=22.5;
xi(6)=23;
xi(7)=23.5;
%xi(8)=24;

    
for i = 1:size(data,1)
   
n_bins(i) =nnz(data(i,:)); 
max_bin = max(n_bins(i));
min_bin = min(n_bins(i));

 if binn(i,7)==1
     mini(i)=23; 
 end
 if binn(i,6)==1
     mini(i)=22.5; 
 end
 if binn(i,5)==1
     mini(i)=22; 
 end
 if binn(i,4)==1
     mini(i)=21.5; 
 end
 if binn(i,3)==1
     mini(i)=21; 
 end
 if binn(i,2)==1
     mini(i)=20.5; 
 end
 if binn(i,1)==1
     mini(i)=missing; 
 end
 
 
 if binn(i,1)==1
     maxi(i)=20.5;  
 end
 if binn(i,2)==1
     maxi(i)=21; 
 end
 if binn(i,3)==1
     maxi(i)=21.5; 
 end
 if binn(i,4)==1
     maxi(i)=22; 
 end
 if binn(i,5)==1
     maxi(i)=22.5; 
 end
 if binn(i,6)==1
     maxi(i)=23; 
 end
 if binn(i,7)==1
     maxi(i)=missing; 
 end

end

% remove guys in 1 bin only
diff=maxi-mini;

sing=sum(binn')';
one_guys=data(sing(:, 1)== 1, :);
one_guys_cum_prob=cum_prob(sing(:, 1)== 1, :);
counter_extreme_guys=0;

N_one_guys=size(one_guys,1);
results_one_guys=zeros(size(one_guys_cum_prob,1),2);
min2=mini(sing(:, 1)== 1, :);
max2=maxi(sing(:, 1)== 1, :);
for i = 1:size(one_guys_cum_prob,1)
    F_i = one_guys_cum_prob(i,:)/100;
    x = xi;
    l_tot = min2(i);
    if ismissing(l_tot)==1
        l_tot=19.5
        counter_extreme_guys=counter_extreme_guys+1;
    end
   
    r_tot = max2(i);
     if ismissing(r_tot)==1
          counter_extreme_guys=counter_extreme_guys+1;
        r_tot=24
    end
   % n = mean(data.n_bins(i1));
   n=1;
    l = min(l_tot);
    r = max(r_tot);
    
    if one_guys(i,:)<40 & l == l_tot & ismissing(r)==0
        l=missing;
    end
     if one_guys(i,:)>60 & r == r_tot & ismissing(r)==0
        l=missing;
     end
    if one_guys(i,:)<40 & r == r_tot & ismissing(l)==0
        r=missing;
    end
     if one_guys(i,:)>60 & l == l_tot & ismissing(l)==0
        r=missing;
     end
     if ismissing(l)==1 & r_tot~=r
        l= (max2(i) - r*sqrt((one_guys(i,:)/100)/2))/(1 - sqrt((one_guys(i,:)/100)/2));
     end
     if ismissing(l)==1 & r_tot==r
        l= (max2(i) - r*sqrt(1-(one_guys(i,:)/100)/2))/(1 - sqrt(1-(one_guys(i,:)/100)/2));
     end
    if ismissing(r)==1 & l_tot==l
        r= (min2(i) - l*sqrt(1-(one_guys(i,:)/100)/2))/(1 - sqrt(1-(one_guys(i,:)/100)/2));
     end
     if ismissing(r)==1 & l_tot~=l
        r= (min2(i) - l*sqrt((one_guys(i,:)/100)/2))/(1 - sqrt((one_guys(i,:)/100)/2));
     end
    
    mean_1guys=(l+r)/2;
    sd_1guys=(r-l)/sqrt(24);
   results_one_guys(i,1)=  mean_1guys;
   results_one_guys(i,2)=  sd_1guys;
   
    
end




% Remove if probabilities are assigned to exactly 2 non-adjacent bins, and
% for guys in 2 bins, fit a triangle as in Engelberg et al. 
two_guys=data(diff(:, 1)== 1, :);
N_two_guys=size(two_guys,1);

two_guys_cum_prob=cum_prob(diff(:, 1)== 1, :);
results_two_guys=zeros(size(two_guys_cum_prob,1),2);
min2=mini(diff(:, 1)== 1, :);
max2=maxi(diff(:, 1)== 1, :);
for i = 1:size(two_guys_cum_prob,1)
    F_i = two_guys_cum_prob(i,:)/100;
    x = xi;
    l_tot = min2(i);
    if ismissing(l_tot)==1
        l_tot=19.5
         counter_extreme_guys=counter_extreme_guys+1;
    end
    r_tot = max2(i);
    if ismissing(r_tot)==1
        r_tot=24
         counter_extreme_guys=counter_extreme_guys+1;
    end
   % n = mean(data.n_bins(i1));
   n=1;
    l = min(l_tot);
    r = max(r_tot);
    
    if two_guys(i,:)<40 & l == l_tot & ismissing(r)==0
        l=missing;
    end
     if two_guys(i,:)>60 & r == r_tot & ismissing(r)==0
        l=missing;
     end
    if two_guys(i,:)<40 & r == r_tot & ismissing(l)==0
        r=missing;
    end
     if two_guys(i,:)>60 & l == l_tot & ismissing(l)==0
        r=missing;
     end
     if ismissing(l)==1 & r_tot~=r
        l= (max2(i) - r*sqrt((two_guys(i,:)/100)/2))/(1 - sqrt((two_guys(i,:)/100)/2));
     end
     if ismissing(l)==1 & r_tot==r
        l= (max2(i) - r*sqrt(1-(two_guys(i,:)/100)/2))/(1 - sqrt(1-(two_guys(i,:)/100)/2));
     end
    if ismissing(r)==1 & l_tot==l
        r= (min2(i) - l*sqrt(1-(two_guys(i,:)/100)/2))/(1 - sqrt(1-(two_guys(i,:)/100)/2));
     end
     if ismissing(r)==1 & l_tot~=l
        r= (min2(i) - l*sqrt((two_guys(i,:)/100)/2))/(1 - sqrt((two_guys(i,:)/100)/2));
     end
    
    mean_2guys=(l+r)/2;
    sd_2guys=(r-l)/sqrt(24);
   results_two_guys(i,1)=  mean_2guys;
   results_two_guys(i,2)=  sd_2guys;
   
    
end


% drop guys in just 1 or 2 bins
diff(isnan(diff))=0;
diff_sing=diff & sing;

%data(diff(:, 1)== 1, :)= [];
data(diff_sing(:, 1)== 1, :)= [];
mini(diff_sing(:, 1)== 1, :)= [];
maxi(diff_sing(:, 1)== 1, :)= [];
cum_prob(diff_sing(:, 1)== 1, :)= [];


y = ((data(:,1))==100);
N_one_guys2=sum(y);
data(y(:, 1)== 1, :)= [];
cum_prob(y(:, 1)== 1, :)= [];
mini(y(:, 1)== 1, :)= [];
maxi(y(:, 1)== 1, :)= [];

% nonparametric analysis, using notation in xlsx file on experts, to match
% numbers


mean_NP=((data(:,1)/100)*20+(data(:,2)/100)*20.75+(data(:,3)/100)*21.25+(data(:,4)/100)*21.75+(data(:,5)/100)*22.25+(data(:,6)/100)*22.75+(data(:,7)/100)*23.5);
AU=((data(:,1)./100).*((20.5^3-(19.5^3))./3)+((data(:,2)./100).*(((21^3)-(20.5^3))./1.5)+((data(:,3)./100).*(((21.5^3)-(21^3))./1.5)+((data(:,4)./100).*(((22^3)-(21.5^3))./1.5)+((data(:,5)./100).*(((22.5^3)-(22^3))./1.5)+((data(:,6)./100).*(((23^3)-(22.5^3))./1.5)+((data(:,7)./100).*(((24^3)-(23^3))./3))))))));
  
AV=((data(:,1)./100).*((20.5^2-(19.5^2))./2)+((data(:,2)./100).*(((21^2)-(20.5^2))./1)+((data(:,3)./100).*(((21.5^2)-(21^2))./1)+((data(:,4)./100).*(((22^2)-(21.5^2))./1)+((data(:,5)./100).*(((22.5^2)-(22^2))./1)+((data(:,6)./100).*(((23^2)-(22.5^2))./1)+((data(:,7)./100).*(((24^2)-(23^2))./2))))))));
AV=AV.^2;


AX=AU-AV-((0.5.^2)/12);
sd_NP=real(sqrt(AX));

mean_q16=(q16_1/100)*(((-23^2)-(-24^2))/2)+(q16_2/100)*(((-22.5^2)-(-23^2))/1)+(q16_3/100)*(((-22^2)-(-22.5^2))/1)+(q16_4/100)*(((-21.5^2)-(-22^2))/1)+(q16_5/100)*(((-21^2)-(-21.5^2))/1)+(q16_6/100)*(((-20.5^2)-(-21^2))/1)+(q16_7/100)*(((-19.5^2)-(-20.5^2))/2);

var1=(q16_1/100)*(((-23^3)-(-24^3))/3)+(q16_2/100)*(((-22.5^3)-(-23^3))/1.5)+(q16_3/100)*(((-22^3)-(-22.5^3))/1.5)+(q16_4/100)*(((-21.5^3)-(-22^3))/1.5)+(q16_5/100)*(((-21^3)-(-21.5^3))/1.5)+(q16_6/100)*(((-20.5^3)-(-21^3))/1.5)+(q16_7/100)*(((-19.5^3)-(-20.5^3))/3);

var_stata=var1-(mean_q16.^2);%-((0.5^2)/12);
sd_stata=sqrt(var_stata);


%mean_NP_2guys=((two_guys(:,1)/100)*-0.5+(two_guys(:,2)/100)*0.25+(two_guys(:,3)/100)*0.75+(two_guys(:,4)/100)*1.25+(two_guys(:,5)/100)*1.75+(two_guys(:,6)/100)*2.25+(two_guys(:,7)/100)*2.75+(two_guys(:,8)/100)*3.5)
%AU=((two_guys(:,1)./100).*((0-(-1^3))./3)+((two_guys(:,2)./100).*(((0.5^3)-(0^3))./1.5)+((two_guys(:,3)./100).*(((1^3)-(0.5^3))./1.5)+((two_guys(:,4)./100).*(((1.5^3)-(1^3))./1.5)+((two_guys(:,5)./100).*(((2^3)-(1.5^3))./1.5)+((two_guys(:,6)./100).*(((2.5^3)-(2^3))./1.5)+((two_guys(:,7)./100).*(((3^3)-(2.5^3))./1.5)+((two_guys(:,8)./100).*(((4^3)-(3^3))./3)))))))));
  
%AV=((two_guys(:,1)./100).*((0^2-(-1^2))./2)+((two_guys(:,2)./100).*(((0.5^2)-(0^2))./1)+((two_guys(:,3)./100).*(((1^2)-(0.5^2))./1)+((two_guys(:,4)./100).*(((1.5^2)-(1^2))./1)+((two_guys(:,5)./100).*(((2^2)-(1.5^2))./1)+((two_guys(:,6)./100).*(((2.5^2)-(2^2))./1)+((two_guys(:,7)./100).*(((3^2)-(2.5^2))./1)+((two_guys(:,8)./100).*(((4^2)-(3^2))./2)))))))));
%AV=AV.^2;


%AX=AU-AV-((0.5.^2)/12);
%sd_NP_2guys=sqrt(AX);



l_max = 20.5-0.001;
        r_min = 23+0.001;
        l_0 = 19.5;
        r_0 = 24;

% set to 1 if running for q5
temp_flag = 0;



% Initialize results matrix
results = zeros(size(data,1),9);resultsN = zeros(size(data,1),4);
fit = zeros(size(data,1),2);test_fit=zeros(size(data,1),2);skewD_i=zeros(size(data,1),2);
bimodal_i=zeros(size(data,1),1);

alpha_0 = 2;
beta_0 = 5; %25;
l_min = 19.6%20.5-2;
r_max = 25;
l_max = 20.5-0.001;
r_min = 23+0.001;
l_0 = 19.5;
r_0 = 24;

% Loop through respondents
for i = 1:size(data,1)
 
    %i1 = strcmp(data.ResponseId, ids(i));
    F_i = cum_prob(i,:)/100;
    x = xi;
    l_tot = mini(i);
    r_tot = maxi(i);
   % n = mean(data.n_bins(i1));
   n=1;
    l = min(l_tot);
    r = max(r_tot);
   
     
    % Case where both bounds are known
    if ismissing(l) ~=1 & ismissing(r)~=1
        lb = [1.01,1.01,l,r];
        lb = [0.01,0.01,l,r];
        ub = [inf,inf,l,r];
        t0 = [alpha_0,beta_0,l,r];
        type = 1;
    end
    % Case where lower bound is unknown
    if ismissing(l)==1 & ismissing(r)~=1
        lb = [1.01,1.01,l_min,r];
        lb = [0.01,0.01,l_min,r];
        ub = [inf,inf,l_max,r];
        t0 = [alpha_0,beta_0,l_0,r];
        type = 2;
    end
    % Case where upper bound is unknown
    if ismissing(l)~=1 & ismissing(r)==1
        lb = [1.01,1.01,l,r_min];
         lb = [0.01,0.01,l,r_min];
        ub = [inf,inf,l,r_max];
        t0 = [alpha_0,beta_0,l,r_0];
        type = 3;
    end
    % Case where both bounds are unknown
    if ismissing(l)==1 & ismissing(r)==1
        lb = [1.01,1.01,l_min,r_min];
        lb = [0.01,0.01,l_min,r_min];
        ub = [inf,inf,l_max,r_max];
        t0 = [alpha_0,beta_0,l_0,r_0];
        type = 4;
    end
    
    lbN=[20.5-2,0.0001];ubN=[25,10];
    options = optimset('MaxFunEvals',50000,'MaxIter',50000);
    % Search for parameters to minimize function almost 0. First do
    % generalized beta, then normal (a la Giordani and Soderlind)
    crit = @(b)sum((betainc4(xi',b(1),b(2),b(3),b(4)) - F_i).^2);
    critN = @(bN)sum((norminc4(xi',bN(1),bN(2)) - F_i).^2);
    [sols,fval] = fminsearchbnd(crit,t0,lb,ub,options);
   % solsN = fminsearchbnd(critN,t0,lbN,ubN,options);xxxx
    [solsN,fvalN] = fminsearch(critN,t0,options);
    resultsN(i,:) = [solsN];
    fit(i,1)=fval; fit(i,2)=fvalN; 
    results(i,1:6) = [sols,n,type];
    eval(i,1) = crit(sols);
    
    test_fit(i,1)=fval; test_fit(i,2)=fvalN;
    best=test_fit(:,1)<test_fit(:,2);
    best=double((best));
    alpha=results(i,1);
beta=results(i,2);
l=results(i,3);
r=results(i,4);

mean_i= (r*alpha + l*beta)/(alpha + beta);
median_i= (r*(alpha - 1/3) + l*(beta - 1/3))/(alpha + beta - 2/3);
sd_i=sqrt((alpha*beta*((r-l)^2))/(((alpha+beta)^2)*(alpha+beta+1)));
results(i,7)=mean_i; results(i,8)=median_i;results(i,9)=sd_i;

skewD_i(i,1)=(2*(1-2*mean_i).*sd_i)./(mean_i*(1-mean_i)+sd_i^2);
skewD_i(i,2)=(2.*(beta-alpha).*sqrt((alpha+beta+1)))./((alpha+beta+2).*sqrt(alpha.*beta)); % prefer this measure.
bimodal_i(i,1)= (alpha<0.99 & beta<0.99);
end

skew=results(:,7)-results(:,8);

compare=[results(:,[7 8 9]) resultsN];


% plots. Stack guys who reply to just 1 or 2 bins below each case

other_guys_mean=[results_one_guys(:,1);results_two_guys(:,1)];
other_guys_sd=[results_one_guys(:,2);results_two_guys(:,2)];

sd_NP=[sd_NP;other_guys_sd];
mean_NP=[mean_NP;other_guys_mean];



t = tiledlayout(3,2);
nexttile
scatter(sd_NP,-mean_NP)
xlabel('\sigma NP')
ylabel('\mu NP')

nexttile
scatter([compare(:,3);other_guys_sd],[-compare(:,1);-other_guys_mean])
xlabel('\sigma genB')
ylabel('\mu genB')
nexttile
scatter([compare(:,5);other_guys_sd],[-compare(:,4);-other_guys_mean])
xlabel('\sigma N')
ylabel('\mu N')

nexttile
scatter(skewD_i(:,2),-compare(:,1))
xlabel('skew genB')
ylabel('\mu genB')


nexttile
scatter(-mean_NP,[-compare(:,1);-other_guys_mean])
xlabel('\mu NP')
ylabel('\mu genB')
refline(1);hline.Color = 'k';

nexttile
scatter(sd_NP,[compare(:,3);other_guys_sd])
xlabel('\sigma NP')
ylabel('\sigma genB')
refline(1);hline.Color = 'k';




% stop code here to plot Figure 1 in main paper

% plot of NP mean and sd for all respondents, incl. those who reply to only
% 1 or 2 bins
scatter(sd_stata(:,1),mean_q16(:,1))

export = table(ids,'VariableNames',{'ResponseId'});
export.alpha = results(:,1);
export.beta = results(:,2);
export.l = results(:,3);
export.r = results(:,4);

writetable(export,'beta_estimation');
