% Import data - variable description below
% ResponseId - unique identifier of survey respondent
% prob - probability assigned to a particular bin
% cum_prob - CDF value at a particular bin
% n_bins - total number of bins with assigned probability
% min - minimum of bin's interval
% max - maximum of bin's interval
% l - global minimum of distribution's support, where known
% r - global maximum of distribution's support, where known
clear all
data = readtable('beta_estimation_input.csv');

% set to 1 if running for q5
temp_flag = 0;

% Sort so that bins are ascending in each respondent
data = sortrows(data,[1,6]);
dates = datenum(datetime(data.date,'InputFormat','ddMMMyyyy'));
ids = unique(data.ResponseId);

% Initialize results matrix
results = zeros(length(ids),6);

alpha_0 = 2;
beta_0 = 5;
l_min = -38;
r_max = 38;
l_max = -12.001;
r_min = 12.001;
l_0 = -13;
r_0 = 13;

% Loop through respondents
for i = 1:length(ids)
    
    i1 = strcmp(data.ResponseId, ids(i));
    F_i = data.cum_prob(i1)/100;
    x = data.max(i1);
    l_tot = data.l(i1);
    r_tot = data.r(i1);
    n = mean(data.n_bins(i1));
    l = min(l_tot);
    r = max(r_tot);
    date = mean(dates(i1));
    
     if date < datenum('07-Apr-2020')
        l_max = -12.001;
        r_min = 12.001;
        l_0 = -13;
        r_0 = 13;
     end
     
     if date >= datenum('07-Apr-2020') & date < datenum('01-May-2020')
        l_max = -20.001;
        r_min = 20.001;
        l_0 = -21;
        r_0 = 21;
     end
    
   if date >= date < datenum('01-May-2020')
        l_max = -25.001;
        r_min = 25.001;
        l_0 = -26;
        r_0 = 26;
     end
  
    % Case where both bounds are known
    if ~isnan(l) & ~isnan(r)
        lb = [1.01,1.01,l,r];
        ub = [inf,inf,l,r];
        t0 = [alpha_0,beta_0,l,r];
        type = 1;
    end
    % Case where lower bound is unknown
    if isnan(l) & ~isnan(r)
        lb = [1.01,1.01,l_min,r];
        ub = [inf,inf,l_max,r];
        t0 = [alpha_0,beta_0,l_0,r];
        type = 2;
    end
    % Case where upper bound is unknown
    if ~isnan(l) & isnan(r)
        lb = [1.01,1.01,l,r_min];
        ub = [inf,inf,l,r_max];
        t0 = [alpha_0,beta_0,l,r_0];
        type = 3;
    end
    % Case where both bounds are unknown
    if isnan(l) & isnan(r)
        lb = [1.01,1.01,l_min,r_min];
        ub = [inf,inf,l_max,r_max];
        t0 = [alpha_0,beta_0,l_0,r_0];
        type = 4;
    end
    options = optimset('MaxFunEvals',50000,'MaxIter',50000);
    % Search for parameters to minimize function almost 0
    crit = @(b)sum((betainc4(x,b(1),b(2),b(3),b(4)) - F_i).^2);
    sols = fminsearchbnd(crit,t0,lb,ub,options);
    results(i,:) = [sols,n,type];
    eval(i,1) = crit(sols);
end

export = table(ids,'VariableNames',{'ResponseId'});
export.alpha = results(:,1);
export.beta = results(:,2);
export.l = results(:,3);
export.r = results(:,4);

writetable(export,'beta_estimation');
