capture log close
log using "2020.log", text replace

clear all
set more off
use "2020_survey.dta"

generate hidgroup_v=1 if hidgroup==1
replace hidgroup_v=2 if hidgroup==2
replace hidgroup_v=3 if hidgroup==3
replace hidgroup_v=4 if hidgroup==5
replace hidgroup_v=5 if hidgroup==6
replace hidgroup_v=6 if hidgroup==7
replace hidgroup_v=7 if hidgroup==4
replace hidgroup_v=8 if hidgroup==8



* measuring certainty (higher value means more certain about value)

generate uncer=1 if q14==7
replace uncer=2 if q14==6
replace uncer=3 if q14==5
replace uncer=4 if q14==4
replace uncer=5 if q14==3
replace uncer=6 if q14==2
replace uncer=7 if q14==1
reg uncer i.hidgroup_v, r

* Probability  interval is 30%
tabulate q15, generate(d_q15)
*correct answer
gen d_intcorr=d_q155
reg d_intcorr i.hidgroup_v, r

*interval
reg int_clean i.hidgroup_v,r

gen inter=q12-q13
replace inter = q13-q12 if inter<0
replace inter =99999 if inter>100
replace inter=. if inter==99999
reg inter i.hidgroup_v,r


* Histogram analysis: compute moments using formula in Clements textbook
* first mean assumes all prob mass at the midpoint. It is in effect sqrt(Clements2) in Expert Survey data.xlsx
gen mean=(q16_1/100)*(((-23^2)-(-24^2))/2)+(q16_2/100)*(((-22.5^2)-(-23^2))/1)+(q16_3/100)*(((-22^2)-(-22.5^2))/1)+(q16_4/100)*(((-21.5^2)-(-22^2))/1)+(q16_5/100)*(((-21^2)-(-21.5^2))/1)+(q16_6/100)*(((-20.5^2)-(-21^2))/1)+(q16_7/100)*(((-19.5^2)-(-20.5^2))/2)
 gen mean_error=-21.7-(-mean)
gen abs_mean_error=abs(mean_error)

* mid point estimate of mean. This is same as mean above
gen mean_mid=(q16_1/100)*-23.5 + (q16_2/100)*-22.75 + (q16_3/100)*-22.25 + (q16_4/100)*-21.75 + (q16_5/100)*-21.25 + (q16_6/100)*-20.75 + (q16_7/100)*-20

* and this is another expression confirming the mean (same as above, except for sign given square)
gen mean_uniform=(q16_1/100)*(((-23^2)-(-24^2))/2)+(q16_2/100)*(((-22.5^2)-(-23^2))/1)+(q16_3/100)*(((-22^2)-(-22.5^2))/1)+(q16_4/100)*(((-21.5^2)-(-22^2))/1)+(q16_5/100)*(((-21^2)-(-21.5^2))/1)+(q16_6/100)*(((-20.5^2)-(-21^2))/1)+(q16_7/100)*(((-19.5^2)-(-20.5^2))/2)

gen var1=(q16_1/100)*(((-23^3)-(-24^3))/3)+(q16_2/100)*(((-22.5^3)-(-23^3))/1.5)+(q16_3/100)*(((-22^3)-(-22.5^3))/1.5)+(q16_4/100)*(((-21.5^3)-(-22^3))/1.5)+(q16_5/100)*(((-21^3)-(-21.5^3))/1.5)+(q16_6/100)*(((-20.5^3)-(-21^3))/1.5)+(q16_7/100)*(((-19.5^3)-(-20.5^3))/3)

* var is variance of histogram using Shepphard correction
gen var=var1-(mean^2)-((0.5^2)/12)
gen sd=sqrt(var)
gen abs_sd_error=abs(0.7-sd)

* distance measure between reported histogram for each individual and true histogram assumed normal with mean of -21.7 and s.d. of 0.7; see histogram.m
gen true1=0.0316
gen true2=0.0949
gen true3=0.2076
gen true4=0.2783
gen true5=0.2289
gen true6=0.1154
gen true7=0.0432

* Cramer-von Mises and KL Distance measures for histograms. Prefer CM, as KL runs into missing values for bins with 0 probabilities
gen CMdistance=((q16_1/100)-true1)^2 + ((q16_2/100)-true2)^2 + ((q16_3/100)-true3)^2 + ((q16_4/100)-true4)^2 + ((q16_5/100)-true5)^2 + ((q16_6/100)-true6)^2 + ((q16_7/100)-true7)^2
gen KLdistance=(q16_1/100)*log((q16_1/100)/true1) + (q16_2/100)*log((q16_2/100)/true2) + (q16_3/100)*log((q16_3/100)/true3) + (q16_4/100)*log((q16_4/100)/true4) + (q16_5/100)*log((q16_5/100)/true5) + (q16_6/100)*log((q16_6/100)/true6) + (q16_7/100)*log((q16_7/100)/true7)


qreg CMdistance i.hidgroup_v, quantile(.25) vce(robust)
qreg CMdistance i.hidgroup_v, quantile(.50) vce(robust)
qreg CMdistance i.hidgroup_v, quantile(.75) vce(robust)



* understanding revisions
*understands revisions
gen d_undrev=q17_2
reg d_undrev i.hidgroup_v, r
*political revisons
gen d_pol=q17_3+q17_4
gen d_polrev=(d_pol>=1)
reg d_polrev i.hidgroup_v, r
*mistakes revisions
gen d_mistrev=q17_1
reg d_mistrev i.hidgroup_v, r
*data limitations
gen d_limit=q17_5
reg d_limit i.hidgroup_v, r




* adding iteraction with know what gdp  is
tabulate q10, generate(d_q10) nofreq
*know what GDP is
gen d_know_gdp=d_q103

*ONS trust
tabulate q9, generate(d_q9) nofreq
*greatly and tend to trust them
gen d_trust_ons=d_q91+d_q92

* trying to replicate james's results
tabulate q8, generate(d_q8) nofreq
*ONS knowledge
gen d_know_ons=d_q81

*the guy that trust ons, know ons and know gdp
gen dchar=d_trust_ons*d_know_ons*d_know_gdp

*heterogenity regressions
reg uncer i.hidgroup_v##dchar, r
reg d_intcorr i.hidgroup_v##dchar, r
reg inter i.hidgroup_v##dchar, r
reg d_undrev i.hidgroup_v##dchar, r
reg d_polrev i.hidgroup_v##dchar, r
reg d_mistrev i.hidgroup_v##dchar, r
reg d_limit i.hidgroup_v##dchar, r
qreg CMdistance i.hidgroup_v##dchar, quantile(.25) vce(robust)
qreg CMdistance i.hidgroup_v##dchar, quantile(.50) vce(robust)
qreg CMdistance i.hidgroup_v##dchar, quantile(.75) vce(robust)

*only for the ones that don't lnow
gen dchar2=1-dchar
reg uncer i.hidgroup_v if dchar2==1, r
reg d_intcorr i.hidgroup_v if dchar2==1, r
reg inter i.hidgroup_v if dchar2==1, r
reg d_undrev i.hidgroup_v if dchar2==1, r
reg d_polrev i.hidgroup_v if dchar2==1, r
reg d_mistrev i.hidgroup_v if dchar2==1, r
reg d_limit i.hidgroup_v if dchar2==1, r


*table for control group

sum uncer d_intcorr inter d_undrev d_polrev d_mistrev d_limit CMdistance if hidgroup_v==1

log close
